//==============================================================================
//
// Title:       Generating IQ Modulated Signal Example
// Purpose:     Example of using instrument driver to control IQ modulated signal generator.
//
//==============================================================================

//==============================================================================
// Include files

#include "rssgs.h"
#include <ansi_c.h>
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rssgs_generating_iq_modulated_signal_example.h"
#include "toolbox.h"

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else

//==============================================================================
// Constants
#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW 

//==============================================================================
// Static global variables
static int panelHandle;
static ViSession instrSession=VI_NULL;

//==============================================================================
// Global functions
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

/// HIFN The main entry-point function.
int main (int argc, char *argv[])
{
    int error = 0;
    
    /* initialize and load resources */
    nullChk (InitCVIRTE (0, argv, 0));
    errChk (panelHandle = LoadPanel (0, "rssgs_generating_iq_modulated_signal_example.uir", PANEL));
    
    /* display the panel and run the user interface */
    errChk (DisplayPanel (panelHandle));
    errChk (RunUserInterface ());

Error:
    /* clean up */
    DiscardPanel (panelHandle);
    return 0;
}

/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rssgs_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rssgs_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}

/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){
    SetCtrlAttribute (panelHandle, PANEL_Q_OFFSET,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_QUAD_OFFSET, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_GIMBA,     ATTR_DIMMED, !enable);   
    SetCtrlAttribute (panelHandle, PANEL_I_OFFSET,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_LEVEL,     ATTR_DIMMED, !enable);   
    SetCtrlAttribute (panelHandle, PANEL_RF_FREQ,   ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RF_OUT,    ATTR_DIMMED, !enable);
    
    if (instrSession == VI_NULL)
        enable = 0;
    else
        enable = 1;
    
    SetCtrlAttribute (panelHandle, PANEL_APPLY,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CONNECT,   ATTR_DIMMED,  enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,      ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,    ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC, ATTR_DIMMED, enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    
    if (instrSession != VI_NULL) 
    {
        if (deviceClear)
        {
            viClear (instrSession);
            viPrintf (instrSession, "*CLS\n");
        }
    
    /* close instrument session */
    rssgs_close (instrSession);
    instrSession = VI_NULL;
    }
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}

//==============================================================================
// UI callback function prototypes

/// HIFN Exit when the user dismisses the panel.
int CVICALLBACK panelCB (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)
    {
        cleanup (VI_TRUE);
        
        QuitUserInterface (0);
    }
    return 0;
}

/// HIFN Configures generator settings.
int CVICALLBACK applyBtn (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64    rfFreq  =   0.0;
    ViReal64    level   =   0.0;
    ViReal64    iOffset =   0.0;
    ViReal64    qOffset =   0.0;
    ViReal64    gainImba=   0.0;
    ViReal64    quadOffset= 0.0;
    ViBoolean   rfState =   VI_FALSE;
    
    ViStatus    status  =   VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:
            /* lock GUI */
            SetWaitCursor (1);
            EnableButtons(VI_FALSE);
            
            /* get control values */
            GetCtrlVal (panelHandle, PANEL_Q_OFFSET,    &qOffset);
            GetCtrlVal (panelHandle, PANEL_QUAD_OFFSET, &quadOffset); //
            GetCtrlVal (panelHandle, PANEL_GIMBA,       &gainImba);   
            GetCtrlVal (panelHandle, PANEL_I_OFFSET,    &iOffset);
            GetCtrlVal (panelHandle, PANEL_LEVEL,       &level);   
            GetCtrlVal (panelHandle, PANEL_RF_FREQ,     &rfFreq);
            GetCtrlVal (panelHandle, PANEL_RF_OUT,      &rfState);
            
            
            /* generator setup, see function help for more details */
            
            CHECKERR (rssgs_SetOperationMode (instrSession, RSSGS_OPERATION_MODE_NORMAL));
            
            CHECKERR (rssgs_SetFrequencyAndLevel (instrSession, rfFreq, level));
            
            CHECKERR (rssgs_SetROSCSource (instrSession, RSSGS_SOURCE_INT)); 
                
            CHECKERR (rssgs_SetFrequencyLocalOscillatorSource (instrSession, RSSGS_SOURCE_INT));
                
            CHECKERR (rssgs_SetIQImpairmentsIOffset (instrSession, iOffset));
                
            CHECKERR (rssgs_SetIQImpairmentsQOffset (instrSession, qOffset));
            
            CHECKERR (rssgs_SetIQImpairmentsGainImbalance (instrSession, gainImba));
            
            CHECKERR (rssgs_GetIQImpairmentsGainImbalance (instrSession, &gainImba));
            
            SetCtrlVal (panelHandle, PANEL_GIMBA_R, gainImba);
            
            CHECKERR (rssgs_SetIQImpairmentsQuadratureOffset (instrSession, quadOffset));
            
            CHECKERR (rssgs_SetIQWideband (instrSession, VI_TRUE));
            
            CHECKERR (rssgs_SetIQImpairmentsState (instrSession, VI_TRUE));
            
            CHECKERR (rssgs_SetRFState(instrSession, rfState));
            
            
            /* unlock GUI */
            EnableButtons(VI_TRUE);
            SetWaitCursor (0);
            break;
    }
    return 0;
}

int CVICALLBACK stopBtn (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            cleanup (VI_FALSE);
            break;
    }
    return 0;
}

/// HIFN Initiates connection to the instrument.
int CVICALLBACK connectBtn (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:
            /* lock GUI */
            SetWaitCursor (1);
            EnableButtons(VI_FALSE);
            
            /* get control values */
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            
            /* initialize driver */
            CHECKERR (rssgs_init (instr_desc, id, reset, &instrSession));
            
            /* initial configuration  */
            applyBtn (panel, PANEL_CONNECT, EVENT_COMMIT, NULL, 0, 0);
        
            /* unlock GUI */
            EnableButtons(VI_TRUE);
            SetWaitCursor (0);
            break;
    }
    return 0;
}
